package istia.st.avat.exemples.actions;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.RejectedExecutionException;

import istia.st.avat.android.Action;
import istia.st.avat.android.Task;
import istia.st.avat.core.IBoss;
import istia.st.avat.core.IWorker;
import istia.st.avat.exemples.activity.Factory;

public class Action_01 extends Action {

	// la liste des nombres aléatoires générés par les tâches lancées
	List<Integer> aleas = new ArrayList<Integer>();

	@Override
	// exécution de l'action
	public void doWork(Object... params) {
		// on crée des tâches asynchrones pour générer les nombres aléatoires
		int nbAleas = ((Integer) params[0]).intValue();
		for (int i = 0; i < nbAleas; i++) {
			// on demande la tâche à la Factory
			Task task = (Task) factory.getObject(Factory.TASK_01, this, String.format("[%s]Task_01-%02d", bossId, i));
			try {
				// on exécute la tâche
				task.doWork(params[1], params[2], params[3]);
			} catch (RejectedExecutionException ex) {
				// on passe la tâche à l'état TERMINATED
				notifyEvent(task, IBoss.WORK_TERMINATED,null);
				// elle a été rejetée - on passe l'exception au boss
				boss.notifyEvent(this, IBoss.WORK_INFO, ex);
			}
		}
		// on commence le monitoring de la fin des tâches lancées
		beginMonitoring();
	}

	@Override
	public void notifyEndOfTasks() {
		// fin des tâches - on fait la somme des nombres reçus
		int somme = 0;
		for (int alea : aleas) {
			somme += alea;
		}
		// on rend cette somme au boss
		boss.notifyEvent(this, IBoss.WORK_TERMINATED, Integer.valueOf(somme));
	}

	// notification d'un événement
	@Override
	public void notifyEvent(IWorker worker, int eventType, Object event) {
		// on passe l'événement au parent
		super.notifyEvent(worker, eventType, event);
		// on passe l'événement INFO au boss
		if (eventType == IBoss.WORK_INFO) {
			boss.notifyEvent(this, eventType, event);
			// si l'info est un entier, on le mémorise
			if (event instanceof Integer) {
				aleas.add((Integer) event);
			}
		}
	}

}
