package istia.st.avat.exemples.tasks;

import istia.st.avat.android.Task;
import istia.st.avat.core.IBoss;
import istia.st.avat.exemples.activity.ClientException;

import java.util.Random;

public class Task_01 extends Task {

	// info renvoyée par la tâche
	private Object info;

	@Override
	public void onPreExecute(){
		// on commence le travail
		boss.notifyEvent(this, IBoss.WORK_STARTED, null);
	}
	
	@Override
	protected Void doInBackground(Object... params) {
		// il faut trois paramètres
		if (params.length != 3) {
			info = new ClientException("Il faut trois paramètres");
			return null;
		}
		// on récupère les paramètres
		int sleepTime = 0;
		int a = 0;
		int b = 0;
		try {
			sleepTime = (Integer) params[2];
			a = (Integer) params[0];
			b = (Integer) params[1];
		} catch (Exception ex) {
			info = new ClientException(String.format("Paramètres incorrects [%d,  %d, %d]", sleepTime, a, b));
			return null;
		}
		// attente
		try {
			Thread.sleep(sleepTime);
		} catch (InterruptedException e) {
			info = e;
			return null;
		}
		// on génère une exception aléatoire 1 fois / 3
		Random random = new Random();
		int i = random.nextInt(3);
		if (i == 0) {
			info = new ClientException("Exception aléatoire");
			return null;
		}
		// sinon on rend un nombre aléatoire entre deux bornes [a,b]
		if (a > b) {
			info = new ClientException(String.format("%d doit être >= à %d", b, a));
			return null;
		}
		info = a + random.nextInt(b - a + 1);
		return null;
	}

	@Override
	// traitement fin de tâche dans le thred de l'UI
	protected void onPostExecute(Void result) {
		// on passe l'info au boss
		boss.notifyEvent(this, IBoss.WORK_INFO, info);
		// on a terminé le travail
		boss.notifyEvent(this, IBoss.WORK_TERMINATED, null);
	}

}
