package istia.st.avat.exemples.vues;

import istia.st.avat.android.Vue;
import istia.st.avat.core.IAction;
import istia.st.avat.core.IBoss;
import istia.st.avat.core.IWorker;
import istia.st.avat.exemples.R;
import istia.st.avat.exemples.activity.Factory;

import java.util.ArrayList;
import java.util.List;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;

public class Vue_01 extends Vue {

	// son identité
	private String bossId;

	// les éléments de l'interface visuelle
	private ListView listRéponses;
	private Button btnExecuter;
	private Button btnAnnuler;
	private EditText edtNbAleas;
	private EditText edtA;
	private EditText edtB;
	private EditText edtSleepTime;
	private TextView txtErrorAleas;
	private TextView txtErrorIntervalle;
	private TextView txtErrorSleepTime;

	// liste des réponses à une commande
	private List<String> réponses = new ArrayList<String>();
	// les saisies
	private int nbAleas;
	private int a;
	private int b;
	private int sleepTime;
	// les evts
	private int nbExceptions = 0;
	private int nbAleasReçus = 0;

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		// on crée la vue du fragment à partir de sa définition XML
		return inflater.inflate(R.layout.vue_01, container, false);
	}

	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		// parent
		super.onActivityCreated(savedInstanceState);

		// zones de saisie
		edtNbAleas = (EditText) activity.findViewById(R.id.edt_nbaleas);
		edtA = (EditText) activity.findViewById(R.id.edt_a);
		edtB = (EditText) activity.findViewById(R.id.edt_b);
		edtSleepTime = (EditText) activity.findViewById(R.id.edt_sleepTime);

		// les messages d'erreur
		txtErrorAleas = (TextView) activity.findViewById(R.id.txt_errorNbAleas);
		txtErrorIntervalle = (TextView) activity.findViewById(R.id.txt_errorIntervalle);
		txtErrorSleepTime = (TextView) activity.findViewById(R.id.txt_errorSleepTime);

		// au départ pas de messages d'erreur
		txtErrorAleas.setText("");
		txtErrorIntervalle.setText("");
		txtErrorSleepTime.setText("");

		// bouton Exécuter
		btnExecuter = (Button) activity.findViewById(R.id.btn_Executer);
		btnExecuter.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View arg0) {
				doExecuter();
			}
		});

		// bouton Annuler
		btnAnnuler = (Button) activity.findViewById(R.id.btn_Annuler);
		btnAnnuler.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View arg0) {
				// on annule l'action
				cancelAll();
				// et l'attente
				cancelWaiting();
			}
		});

		// réponses des actions et tâches
		listRéponses = (ListView) activity.findViewById(R.id.lst_reponses);
		// état des boutons
		btnExecuter.setVisibility(View.VISIBLE);
		btnAnnuler.setVisibility(View.INVISIBLE);
	}

	protected void doExecuter() {
		// on efface les éventuels msg d'erreur précédents
		txtErrorAleas.setText("");
		txtErrorIntervalle.setText("");
		txtErrorSleepTime.setText("");
		// on teste la validité des saisies
		if (!isPageValid()) {
			return;
		}
		// on efface les réponses précédentes
		réponses.clear();
		// on exécute Action_01 de façon asynchrone
		IAction action = (IAction) factory.getObject(Factory.ACTION_01, this, "Action_01");
		// on signale au parent le démarrage de l'action
		super.notifyEvent(action, IBoss.WORK_STARTED, null);
		// on commence à attendre
		beginWaiting();
		// raz compteurs
		nbAleasReçus = 0;
		nbExceptions = 0;
		// on exécute l'action (nbAleas,a,b, sleepTime)
		action.doWork(Integer.parseInt(edtNbAleas.getText().toString()), Integer.parseInt(edtA.getText().toString()),
				Integer.parseInt(edtB.getText().toString()), Integer.parseInt(edtSleepTime.getText().toString()));
		// début du monitoring
		beginMonitoring();
	}

	// on vérifie la validité des données saisies
	private boolean isPageValid() {
		// saisie du nombre de nombres aléatoires
		nbAleas = 0;
		Boolean erreur = false;
		int nbErreurs = 0;
		try {
			nbAleas = Integer.parseInt(edtNbAleas.getText().toString());
			erreur = (nbAleas < 1);
		} catch (Exception ex) {
			erreur = true;
		}
		// erreur ?
		if (erreur) {
			nbErreurs++;
			txtErrorAleas.setText(R.string.txt_errorNbAleas);
		}
		// saisie de a
		a = 0;
		erreur = false;
		try {
			a = Integer.parseInt(edtA.getText().toString());
		} catch (Exception ex) {
			erreur = true;
		}
		// erreur ?
		if (erreur) {
			nbErreurs++;
			txtErrorIntervalle.setText(R.string.txt_errorIntervalle);
		}
		// saisie de b
		b = 0;
		erreur = false;
		try {
			b = Integer.parseInt(edtB.getText().toString());
			erreur = b < a;
		} catch (Exception ex) {
			erreur = true;
		}
		// erreur ?
		if (erreur) {
			nbErreurs++;
			txtErrorIntervalle.setText(R.string.txt_errorIntervalle);
		}
		// saisie de sleepTime
		sleepTime = 0;
		erreur = false;
		try {
			sleepTime = Integer.parseInt(edtSleepTime.getText().toString());
			erreur = (sleepTime < 0);
		} catch (Exception ex) {
			erreur = true;
		}
		// erreur ?
		if (erreur) {
			nbErreurs++;
			txtErrorSleepTime.setText(R.string.txt_errorSleepTime);
		}
		// retour
		return (nbErreurs == 0);
	}

	@Override
	public void notifyEndOfTasks() {
		// fin de l'attente
		cancelWaiting();
	}

	@Override
	public void notifyEvent(IWorker worker, int eventType, Object event) {
		// on passe l'évt à la classe parent
		super.notifyEvent(worker, eventType, event);
		// on gère l'évt WORK_INFO
		if (eventType == IBoss.WORK_INFO) {
			// exception ?
			if (event instanceof Exception) {
				réponses.add(0, ((Exception) event).getMessage());
				nbExceptions++;
			} else if (event instanceof Integer) {
				réponses.add(0, event.toString());
				nbAleasReçus++;
			}
		} else {
			// on gère l'évt WORK_TERMINATED
			if (eventType == IBoss.WORK_TERMINATED && event instanceof Integer) {
				réponses.add(0, String.format("Nb Exceptions=%d, Nb Aléatoires=%d, Somme des nombres reçus = %d", nbExceptions,
						nbAleasReçus, (Integer) event));
			}
		}
		// rafraîchissement des réponses
		listRéponses.setAdapter(new ArrayAdapter<String>(activity, android.R.layout.simple_list_item_1, android.R.id.text1,
				réponses));
	}

	// début de l'attente
	private void beginWaiting() {
		// on met le sablier
		activity.setProgressBarIndeterminateVisibility(true);
		// état des boutons
		btnExecuter.setVisibility(View.INVISIBLE);
		btnAnnuler.setVisibility(View.VISIBLE);

	}

	// fin de l'attente
	protected void cancelWaiting() {
		// on cache le sablier
		activity.setProgressBarIndeterminateVisibility(false);
		// état des boutons
		btnExecuter.setVisibility(View.VISIBLE);
		btnAnnuler.setVisibility(View.INVISIBLE);
	}

	@Override
	public void setBossId(String id) {
		this.bossId = id;

	}

	@Override
	public String getBossId() {
		return bossId;
	}

}
