package istia.st.avat.exemples.tasks;

import istia.st.avat.android.Task;
import istia.st.avat.client.IMetier;
import istia.st.avat.client.dao.ClientException;
import istia.st.avat.core.IBoss;
import istia.st.avat.exemples.activity.Factory;

public class Task_01 extends Task {

	// info renvoyée par la tâche
	private Object info;

	@Override
	public void onPreExecute(){
		// on commence le travail
		boss.notifyEvent(this, IBoss.WORK_STARTED, null);
	}
	
	@Override
	// on exécute la tâche (urlSrviceRest, a, b, sleepTime)
	protected Void doInBackground(Object... params) {
		// il faut quatre paramètres
		if (params.length != 4) {
			info = new ClientException("Il faut quatre paramètres");
			return null;
		}
		// on récupère les paramètres
		String urlServiceRest = null;
		int a = 0;
		int b = 0;
		int sleepTime = 0;
		
		try {
			urlServiceRest = (String) params[0];
			a = (Integer) params[1];
			b = (Integer) params[2];
			sleepTime = (Integer) params[3];			
		} catch (Exception ex) {
			info = new ClientException(String.format("Paramètres incorrects [%s,  %d, %d, %d]", urlServiceRest, a, b, sleepTime));
			return null;
		}
		// vérification des bornes [a,b]
		if (a > b) {
			info = new ClientException(String.format("%d doit être >= à %d", b, a));
			return null;
		}
		// attente
		try {
			Thread.sleep(sleepTime);
		} catch (InterruptedException e) {
			info = e;
			return null;
		}		
		// on demande le nombre à la couche métier
		IMetier metier = (IMetier) factory.getObject(Factory.METIER, (Object[]) null);
		try {
			info = metier.randomNumber(urlServiceRest, a, b);
		} catch (Exception ex) {
			info = ex;
		}
		// fin
		return null;
	}

	@Override
	// traitement fin de tâche dans le thread de l'UI
	protected void onPostExecute(Void result) {
		// on passe l'info au boss
		boss.notifyEvent(this, IBoss.WORK_INFO, info);
		// on a terminé le travail
		boss.notifyEvent(this, IBoss.WORK_TERMINATED, null);
	}

}
