package istia.st.avat.exemples.activity;

import istia.st.avat.android.Team;
import istia.st.avat.android.Vue;
import istia.st.avat.core.IAction;
import istia.st.avat.core.IBoss;
import istia.st.avat.core.IFactory;
import istia.st.avat.core.ITeam;
import istia.st.avat.exemples.actions.Action_01;
import istia.st.avat.exemples.vues.Vue_01;
import android.app.Activity;

public class Factory implements IFactory {

	// constantes
	public static final int CONFIG = -1;
	public static final int ACTION_01 = 3;
	public static final int VUE_01 = 4;

	// ------------------------------------- DEBUT SINGLETONS
	// la configuration
	private Config config;
	private boolean verbose;

	// les vues
	private Vue vue_01;

	// l'activité principale
	private Activity activity;

	// ------------------------------------- FIN SINGLETONS

	// constructeur
	public Factory(Activity activity, Config config) {
		// l'activité
		this.activity = activity;
		// la configuration
		this.config = config;
		// le mode verbeux ou non
		verbose = config.isVerbose();
	}

	@Override
	public Object getObject(int id, Object... params) {
		switch (id) {
		case CONFIG:
			return config;
		case ACTION_01:
			return getAction_01(params);
		case VUE_01:
			return getVue_01(params);
		}
		return null;
	}

	private Object getVue_01(Object... params) {
		// la vue est un singleton
		if (vue_01 == null) {
			// la vue
			vue_01 = new Vue_01();
			// sa factory
			vue_01.setFactory(this);
			// son équipe
			ITeam team = new Team();
			team.setMonitor(vue_01);
			vue_01.setTeam(team);
			// son activité
			vue_01.setActivity(activity);
			// son mode verbeux ou non
			vue_01.setVerbose(verbose);
			// son id
			vue_01.setBossId((String) params[0]);
		}
		// on rend la référence
		return vue_01;
	}

	// une action reçoit deux paramètres : IBoss, id
	private Object getAction_01(Object[] params) {
		// instanciation
		IAction action = new Action_01();
		// initialisation
		// le boss
		action.setBoss((IBoss) params[0]);
		// l'équipe
		ITeam team = new Team();
		team.setMonitor(action);
		action.setTeam(team);
		// l'identité
		action.setWorkerId((String) params[1]);
		action.setBossId((String) params[1]);
		// la factory
		action.setFactory(this);
		// le mode verbeux ou non
		action.setVerbose(verbose);
		// fin
		return action;
	}

}
