package istia.st.avat.exemples.vues;

import istia.st.avat.android.Vue;
import istia.st.avat.core.IAction;
import istia.st.avat.core.IBoss;
import istia.st.avat.core.ISession;
import istia.st.avat.core.IWorker;
import istia.st.avat.exemples.R;
import istia.st.avat.exemples.activity.Factory;
import istia.st.avat.exemples.activity.MainActivity;

import java.net.URI;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

public class Vue_01 extends Vue {

	// les éléments de l'interface visuelle
	private Button btnValider;
	private EditText edtNbAleas;
	private EditText edtA;
	private EditText edtB;
	private EditText edtUrlServiceRest;
	private EditText edtSleepTime;
	private TextView txtErrorAleas;
	private TextView txtErrorIntervalle;
	private TextView txtErrorUrlServiceRest;
	private TextView txtErrorSleepTime;

	// les saisies
	private int nbAleas;
	private int a;
	private int b;
	private String urlServiceRest;
	private int sleepTime;

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		// on crée la vue du fragment à partir de sa définition XML
		return inflater.inflate(R.layout.vue_01, container, false);
	}

	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		// parent
		super.onActivityCreated(savedInstanceState);

		// zones de saisie
		edtUrlServiceRest = (EditText) activity.findViewById(R.id.edt_urlServiceRest);
		edtNbAleas = (EditText) activity.findViewById(R.id.edt_nbaleas);
		edtA = (EditText) activity.findViewById(R.id.edt_a);
		edtB = (EditText) activity.findViewById(R.id.edt_b);
		edtSleepTime = (EditText) activity.findViewById(R.id.edt_sleepTime);

		// les messages d'erreur
		txtErrorAleas = (TextView) activity.findViewById(R.id.txt_errorNbAleas);
		txtErrorIntervalle = (TextView) activity.findViewById(R.id.txt_errorIntervalle);
		txtErrorUrlServiceRest = (TextView) activity.findViewById(R.id.txt_errorUrlServiceRest);
		txtErrorSleepTime = (TextView) activity.findViewById(R.id.txt_errorSleepTime);

		// au départ pas de messages d'erreur
		txtErrorAleas.setText("");
		txtErrorIntervalle.setText("");
		txtErrorUrlServiceRest.setText("");
		txtErrorSleepTime.setText("");

		// bouton Valider
		btnValider = (Button) activity.findViewById(R.id.btn_Valider);
		btnValider.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View arg0) {
				doValider();
			}
		});

	}

	protected void doValider() {
		// on efface les éventuels msg d'erreur précédents
		txtErrorAleas.setText("");
		txtErrorIntervalle.setText("");
		txtErrorUrlServiceRest.setText("");
		txtErrorSleepTime.setText("");

		// on teste la validité des saisies
		if (!isPageValid()) {
			return;
		}
		// on exécute Action_01 de façon asynchrone
		IAction action = (IAction) factory.getObject(Factory.ACTION_01, this, "Action_01");
		// on signale au parent le démarrage de l'action
		super.notifyEvent(action, WORK_STARTED, null);		
		// on exécute l'action (urlServiceRest,nbAleas,a,b, sleepTime)
		action.doWork(edtUrlServiceRest.getText().toString(), Integer.parseInt(edtNbAleas.getText().toString()),
				Integer.parseInt(edtA.getText().toString()), Integer.parseInt(edtB.getText().toString()),
				Integer.parseInt(edtSleepTime.getText().toString()));
		// on monitore
		beginMonitoring();
	}

	private boolean isPageValid() {
		// on vérifie la validité des données saisies
		Boolean erreur = false;
		int nbErreurs = 0;
		// validité de l'URL du service REST
		urlServiceRest = edtUrlServiceRest.getText().toString().trim();
		try {
			new URI(String.format("http://%s", urlServiceRest));
		} catch (Exception ex) {
			// affichage erreur
			txtErrorUrlServiceRest.setText(R.string.txt_errorUrlServiceRest);
		}
		// saisie du nombre de nombres aléatoires
		nbAleas = 0;
		try {
			nbAleas = Integer.parseInt(edtNbAleas.getText().toString());
			erreur = (nbAleas < 1);
		} catch (Exception ex) {
			erreur = true;
		}
		// erreur ?
		if (erreur) {
			nbErreurs++;
			txtErrorAleas.setText(R.string.txt_errorNbAleas);
		}
		// saisie de a
		a = 0;
		erreur = false;
		try {
			a = Integer.parseInt(edtA.getText().toString());
		} catch (Exception ex) {
			erreur = true;
		}
		// erreur ?
		if (erreur) {
			nbErreurs++;
			txtErrorIntervalle.setText(R.string.txt_errorIntervalle);
		}
		// saisie de b
		b = 0;
		erreur = false;
		try {
			b = Integer.parseInt(edtB.getText().toString());
			erreur = b < a;
		} catch (Exception ex) {
			erreur = true;
		}
		// erreur ?
		if (erreur) {
			nbErreurs++;
			txtErrorIntervalle.setText(R.string.txt_errorIntervalle);
		}
		// saisie de sleepTime
		sleepTime = 0;
		erreur = false;
		try {
			sleepTime = Integer.parseInt(edtSleepTime.getText().toString());
			erreur = (sleepTime < 0);
		} catch (Exception ex) {
			erreur = true;
		}
		// erreur ?
		if (erreur) {
			nbErreurs++;
			txtErrorSleepTime.setText(R.string.txt_errorSleepTime);
		}

		// retour
		return (nbErreurs == 0);
	}

	@Override
	public void notifyEndOfTasks() {
		// on affiche la vue 2
		((MainActivity) activity).showVue(2);		
	}

	@Override
	public void notifyEvent(IWorker worker, int eventType, Object event) {
		// parent
		super.notifyEvent(worker, eventType, event);
		// on gère l'évt WORK_TERMINATED
		if (eventType == IBoss.WORK_TERMINATED) {
			// on met les résultats dans la session (urlServiceRest, nbAleas, a, b,
			// sleepTime)
			ISession session = (ISession) factory.getObject(Factory.SESSION, (Object[]) null);
			Object[] results = (Object[]) event;
			session.add("urlServiceRest", results[0]);
			session.add("nbAleas", results[1]);
			session.add("a", results[2]);
			session.add("b", results[3]);
			session.add("sleepTime", results[4]);
		}
	}

}
