package android.rdvmedecins.ui.activity;

import istia.st.avat.android.Team;
import istia.st.avat.android.Vue;
import istia.st.avat.core.IBoss;
import istia.st.avat.core.IFactory;
import istia.st.avat.core.ISession;
import istia.st.avat.core.ITask;
import istia.st.avat.core.ITeam;
import android.app.Activity;
import android.rdvmedecins.dao.Dao;
import android.rdvmedecins.dao.IDao;
import android.rdvmedecins.metier.IMetier;
import android.rdvmedecins.metier.Metier;
import android.rdvmedecins.ui.tasks.AjouterRvTask;
import android.rdvmedecins.ui.tasks.GetAgendaMedecinTask;
import android.rdvmedecins.ui.tasks.GetAllClientsTask;
import android.rdvmedecins.ui.tasks.GetAllMedecinsTask;
import android.rdvmedecins.ui.tasks.SupprimerRvTask;
import android.rdvmedecins.ui.vues.AccueilVue;
import android.rdvmedecins.ui.vues.AgendaVue;
import android.rdvmedecins.ui.vues.AjoutRvVue;
import android.rdvmedecins.ui.vues.ConfigVue;

public class Factory implements IFactory {

	// constantes
	public static final int CONFIG = -1;
	public static final int SESSION = -2;
	public static final int METIER = -3;
	public static final int CONFIG_VUE = 10;
	public static final int ACCUEIL_VUE = 11;
	public static final int AGENDA_VUE = 12;
	public static final int AJOUT_RV_VUE = 13;
	public static final int GETALLMEDECINS_TASK = 30;
	public static final int GETALLCLIENTS_TASK = 31;
	public static final int GETAGENDAMEDECIN_TASK = 32;
	public static final int AJOUTER_RV_TASK = 33;
	public static final int SUPPRIMER_RV_TASK = 34;

	// ------------------------------------- DEBUT SINGLETONS
	// la configuration
	private Config config;
	private boolean verbose;
	private int timeout;

	// la couche métier
	private IMetier metier;

	// la session
	private ISession session;

	// les vues
	private Vue configVue;
	private Vue accueilVue;
	private Vue agendaVue;
	private Vue ajoutRvVue;

	// l'activité principale
	private Activity activity;

	// ------------------------------------- FIN SINGLETONS

	// constructeur
	public Factory(Activity activity, Config config) {
		// l'activité
		this.activity = activity;
		// la configuration
		this.config = config;
		// le mode verbeux ou non
		verbose = config.isVerbose();
		// le temps d'attente de la réponse du service REST en millisecondes
		timeout = config.getTimeout();
	}

	@Override
	public Object getObject(int id, Object... params) {
		switch (id) {
		case CONFIG:
			return config;
		case SESSION:
			return getSession();
		case METIER:
			return getMetier(params);
		case CONFIG_VUE:
			return getConfigVue(params);
		case ACCUEIL_VUE:
			return getAccueilVue(params);
		case AGENDA_VUE:
			return getAgendaVue(params);
		case AJOUT_RV_VUE:
			return getAjoutRvVue(params);
		case GETALLMEDECINS_TASK:
			return getAllMedecinsTask(params);
		case GETALLCLIENTS_TASK:
			return getAllClientsTask(params);
		case GETAGENDAMEDECIN_TASK:
			return getAgendaMedecinTask(params);
		case AJOUTER_RV_TASK:
			return getAjouterRvTask(params);
		case SUPPRIMER_RV_TASK:
			return getSupprimerRvTask(params);
		}
		return null;
	}

	// une tâche reçoit deux paramètres : IBoss, id
	private Object getAgendaMedecinTask(Object[] params) {
		// instanciation
		ITask task = new GetAgendaMedecinTask();
		// initialisation
		init(task, params);
		// fin
		return task;
	}

	// une tâche reçoit deux paramètres : IBoss, id
	private Object getAllMedecinsTask(Object... params) {
		// instanciation
		ITask task = new GetAllMedecinsTask();
		// initialisation
		init(task, params);
		// fin
		return task;
	}

	// une tâche reçoit deux paramètres : IBoss, id
	private Object getAllClientsTask(Object... params) {
		// instanciation
		ITask task = new GetAllClientsTask();
		// initialisation
		init(task, params);
		// fin
		return task;
	}

	// une tâche reçoit deux paramètres : IBoss, id
	private Object getAjouterRvTask(Object[] params) {
		// instanciation
		ITask task = new AjouterRvTask();
		// initialisation
		init(task, params);
		// fin
		return task;
	}

	// une tâche reçoit deux paramètres : IBoss, id
	private Object getSupprimerRvTask(Object[] params) {
		// instanciation
		ITask task = new SupprimerRvTask();
		// initialisation
		init(task, params);
		// fin
		return task;
	}
	
	// une tâche reçoit deux paramètres : IBoss, id
	private void init(ITask task, Object... params) {
		// le boss
		task.setBoss((IBoss) params[0]);
		// l'identité
		task.setWorkerId((String) params[1]);
		// la factory
		task.setFactory(this);
	}

	// une vue reçoit 1 paramètre : id
	private Object getConfigVue(Object[] params) {
		// la vue est un singleton
		if (configVue == null) {
			// la vue
			configVue = new ConfigVue();
			// initialisation
			init(configVue, params);
		}
		// on rend la référence
		return configVue;
	}

	// une vue reçoit 1 paramètre : id
	private Object getAccueilVue(Object[] params) {
		// la vue est un singleton
		if (accueilVue == null) {
			// la vue
			accueilVue = new AccueilVue();
			// initialisation
			init(accueilVue, params);
		}
		// on rend la référence
		return accueilVue;
	}

	// une vue reçoit 1 paramètre : id
	private Object getAgendaVue(Object[] params) {
		// la vue est un singleton
		if (agendaVue == null) {
			// la vue
			agendaVue = new AgendaVue();
			// initialisation
			init(agendaVue, params);
		}
		// on rend la référence
		return agendaVue;
	}

	// une vue reçoit 1 paramètre : id
	private Object getAjoutRvVue(Object[] params) {
		// la vue est un singleton
		if (ajoutRvVue == null) {
			// la vue
			ajoutRvVue = new AjoutRvVue();
			// initialisation
			init(ajoutRvVue, params);
		}
		// on rend la référence
		return ajoutRvVue;
	}

	// initilisation d'une vue
	private void init(Vue vue, Object... params) {
		// sa factory
		vue.setFactory(this);
		// son équipe
		ITeam team = new Team();
		team.setMonitor(vue);
		vue.setTeam(team);
		// son activité
		vue.setActivity(activity);
		// son mode verbeux ou non
		vue.setVerbose(verbose);
		// son id
		vue.setBossId((String) params[1]);
	}

	// couche métier
	private Object getMetier(Object[] params) {
		// on instancie les couches [dao] et [metier]
		if (metier == null) {
			// couche DAO
			IDao dao = new Dao();
			dao.setTimeout(timeout);
			// couche [métier]
			metier = new Metier();
			metier.setDao(dao);
		}
		// maj éventuelle de l'Url du service REST
		if (params != null && params.length == 1) {
			metier.setUrlServiceRest((String) params[0]);
		}
		// on rend la référence
		return metier;
	}

	// session de l'activité
	private Object getSession() {
		if (session == null) {
			session = new Session();
		}
		return session;
	}
}
