package android.rdvmedecins.ui.vues;

import istia.st.avat.android.Vue;
import istia.st.avat.core.IBoss;
import istia.st.avat.core.ITask;
import istia.st.avat.core.IWorker;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.concurrent.RejectedExecutionException;

import android.os.Bundle;
import android.rdvmedecins.entities.dao.Medecin;
import android.rdvmedecins.entities.metier.AgendaMedecinJour;
import android.rdvmedecins.ui.R;
import android.rdvmedecins.ui.activity.Factory;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;

public class AccueilVue extends LocalVue {

	// les éléments de l'interface visuelle
	private Button btnValider;
	private Button btnAnnuler;
	private Spinner spinnerMedecins;
	private EditText edtJourRv;
	private TextView txtErrorJourRv;

	// les médecins
	private List<Medecin> medecins;

	// ses résultats
	private List<Object> results = new ArrayList<Object>();

	// les saisies
	private Long idMedecin;
	private String jourRv;
	private boolean canceled;
	
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		// on crée la vue du fragment à partir de sa définition XML
		return inflater.inflate(R.layout.accueil, container, false);
	}

	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		// parent
		super.onActivityCreated(savedInstanceState);

		// spinner
		spinnerMedecins = (Spinner) activity.findViewById(R.id.spinnerMedecins);

		// jour
		edtJourRv = (EditText) activity.findViewById(R.id.edt_JourRv);

		// msg d'erreur
		txtErrorJourRv = (TextView) activity.findViewById(R.id.txt_errorJourRv);

		// bouton Valider
		btnValider = (Button) activity.findViewById(R.id.btn_Valider);
		btnValider.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View arg0) {
				doValider();
			}
		});

		// bouton Annuler
		btnAnnuler = (Button) activity.findViewById(R.id.btn_Annuler);
		btnAnnuler.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View arg0) {
				// on annule les tâches en cours
				cancelAll();
				// on annule l'attente
				cancelWaiting();
				// on note l'annulation
				canceled=true;
			}
		});
		// état des boutons
		btnValider.setVisibility(View.VISIBLE);
		btnAnnuler.setVisibility(View.INVISIBLE);
		// au départ pas d'erreurs
		txtErrorJourRv.setText("");
	}

	@SuppressWarnings("unchecked")
	@Override
	public void onResume() {
		// parent
		super.onResume();
		// on récupère les médecins en session
		medecins = (List<Medecin>) session.get("medecins");
		// on construit le tableau affiché par le spinner
		String[] arrayMedecins = new String[medecins.size()];
		int i = 0;
		for (Medecin medecin : medecins) {
			arrayMedecins[i] = String.format("%s %s %s", medecin.getTitre(), medecin.getPrenom(), medecin.getNom());
			i++;
		}
		// on associe les médecins au spinner
		ArrayAdapter<String> dataAdapter = new ArrayAdapter<String>(activity, android.R.layout.simple_spinner_item,
				arrayMedecins);
		dataAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		spinnerMedecins.setAdapter(dataAdapter);
		// on gère les liens de navigation
		activateLinks(1);
	}

	// validation de la page
	protected void doValider() {
		// on teste la validité des saisies
		if (!isPageValid()) {
			return;
		}
		// on nettoie les résultats
		results.clear();		
		// pas encore d'annulation
		canceled=false;
		// on demande l'agenda du médecin
		ITask getAgendaMedecinTask = (ITask) factory
				.getObject(Factory.GETAGENDAMEDECIN_TASK, this, "AgendaMedecinJourTask");
		try {
			// on exécute la tâche (idMedecin, jourRv)
			getAgendaMedecinTask.doWork(idMedecin, jourRv);
		} catch (RejectedExecutionException ex) {
			// la tâche a été rejetée
			super.notifyEvent(getAgendaMedecinTask, IBoss.WORK_TERMINATED, (Object[]) null);
			// affichage exception
			showException(ex);
			// retour à l'UI
			return;
		}		
		// début de l'attente
		beginWaiting();
		// début monitoring
		beginMonitoring();
	}

	private boolean isPageValid() {
		// on efface les erreurs précédentes
		txtErrorJourRv.setText("");
		// on note l'id du médecin sélectionné
		idMedecin = medecins.get(spinnerMedecins.getSelectedItemPosition()).getId();
		// on vérifie la validité des données saisies
		Boolean erreur = false;
		// vérification format date
		jourRv = edtJourRv.getText().toString();
		if (!jourRv.matches("\\s*\\d{2}-\\d{2}-\\d{4}\\s*")) {
			// on affiche le msg d'erreur
			txtErrorJourRv.setText(R.string.txt_errorJour);
			return false;
		}
		// vérification validité de la date
		try {
			SimpleDateFormat simpleDateFormat = new SimpleDateFormat("dd-MM-yyyy", Locale.FRENCH);
			simpleDateFormat.setLenient(false);
			simpleDateFormat.parse(jourRv);
		} catch (ParseException ex) {
			// on note l'erreur
			erreur = true;
			// on l'affiche
			txtErrorJourRv.setText(R.string.txt_errorJour);
		}
		// retour
		return !erreur;
	}

	@Override
	public void notifyEndOfTasks() {
		// fin de l'attente
		cancelWaiting();
		// tâches annulées ?
		if(canceled){
			return;
		}
		// on gère le résultat
		boolean erreur = false;
		for (Object result : results) {
			// on traite les résultats des tâches exécutées
			if (result instanceof AgendaMedecinJour) {
				// on met l'agenda dans la session
				session.add("agenda", result);
			} else if (result instanceof Exception) {
				// on affiche l'exception
				showException((Exception) result);
				// erreur
				erreur = true;
				break;
			}
		}
		// si pas d'erreur, on passe à la vue suivante
		if (!erreur) {
			Vue agendaVue = (Vue) factory.getObject(Factory.AGENDA_VUE, null, "AgendaVue");
			mainActivity.showVue(agendaVue);
		}
	}

	@Override
	// on reçoit des notifications de l'action
	public void notifyEvent(IWorker worker, int eventType, Object event) {
		// on passe l'info au parent
		super.notifyEvent(worker, eventType, event);
		// on traite le cas WORK_INFO
		if (eventType == IBoss.WORK_INFO) {
			results.add(event);
		}
	}

	// début de l'attente
	private void beginWaiting() {
		// on met le sablier
		showHourGlass();
		// état des boutons
		btnValider.setVisibility(View.INVISIBLE);
		btnAnnuler.setVisibility(View.VISIBLE);

	}

	// fin de l'attente
	protected void cancelWaiting() {
		// on cache le sablier
		hideHourGlass();
		// état des boutons
		btnValider.setVisibility(View.VISIBLE);
		btnAnnuler.setVisibility(View.INVISIBLE);
	}

}
