package android.rdvmedecins.ui.vues;

import istia.st.avat.android.Vue;
import istia.st.avat.core.IBoss;
import istia.st.avat.core.ITask;
import istia.st.avat.core.IWorker;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.concurrent.RejectedExecutionException;

import android.os.Bundle;
import android.rdvmedecins.entities.dao.Medecin;
import android.rdvmedecins.entities.metier.AgendaMedecinJour;
import android.rdvmedecins.entities.metier.CreneauMedecinJour;
import android.rdvmedecins.ui.R;
import android.rdvmedecins.ui.activity.Factory;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;

public class AgendaVue extends LocalVue {

	// les éléments de l'interface visuelle
	private TextView txtTitre2;
	private ListView lstCreneaux;
	private Button btnAnnuler;

	// l'agenda
	private AgendaMedecinJour agenda;

	// les résultats des tâches lancées par la vue
	private List<Object> results = new ArrayList<Object>();

	// mémoires
	private int firstPosition;
	private int top;
	private boolean canceled;

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		// on crée la vue du fragment à partir de sa définition XML
		return inflater.inflate(R.layout.agenda, container, false);
	}

	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		// parent
		super.onActivityCreated(savedInstanceState);
		// le titre
		txtTitre2 = (TextView) activity.findViewById(R.id.txt_titre2_agenda);
		// la liste des créneaux
		lstCreneaux = (ListView) activity.findViewById(R.id.listViewAgenda);
		lstCreneaux.setItemsCanFocus(true);
		// bouton Annuler
		btnAnnuler = (Button) activity.findViewById(R.id.btn_Annuler);
		btnAnnuler.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View arg0) {
				// on annule les tâches en cours
				cancelAll();
				// on annule l'attente
				cancelWaiting();
				// on note l'annulation
				canceled = true;
			}
		});
	}

	@Override
	public void onResume() {
		// parent
		super.onResume();
		// on récupère l'agenda en session
		agenda = (AgendaMedecinJour) session.get("agenda");
		// on génère le titre de la page
		Medecin medecin = agenda.getMedecin();
		String text = String.format("Rendez-vous de %s %s %s le %s", medecin.getTitre(), medecin.getPrenom(),
				medecin.getNom(), new SimpleDateFormat("dd-MM-yyyy", Locale.FRANCE).format(agenda.getJour()));
		txtTitre2.setText(text);
		// on génère la liste des créneaux
		ArrayAdapter<CreneauMedecinJour> adapter = new ListCreneauxAdapter(mainActivity, R.layout.creneau_medecin,
				agenda.getCreneauxMedecinJour(), this);
		lstCreneaux.setAdapter(adapter);
		// on se positionne au bon endroit du ListView
		lstCreneaux.setSelectionFromTop(firstPosition, top);
		// on gère les liens de navigation
		activateLinks(2);
		// état du bouton Annuler
		btnAnnuler.setVisibility(View.INVISIBLE);
	}

	// validation de la page
	protected void doValider(int position, String texte) {
		// on met la position dans la session
		session.add("position", position);
		// on note la position du scroll pour y revenir
		// lire [http://stackoverflow.com/questions/3014089/maintain-save-restore-scroll-position-when-returning-to-a-listview]
		// position du 1er élément visible complètement ou non
		firstPosition = lstCreneaux.getFirstVisiblePosition();
		// offset Y de cet élément par rapport au haut du ListView
		// mesure la hauteur de la partie éventuellement cachée
		View v = lstCreneaux.getChildAt(0);
		top = (v == null) ? 0 : v.getTop();
		// selon le texte, on ne fait pas la même chose
		if (texte.equals(getResources().getString(R.string.btn_ajouter))) {
			doAjouter(position);
		} else {
			doSupprimer(position);
		}
	}

	private void doSupprimer(int position) {
		// on nettoie les résultats
		results.clear();
		// pas encore d'annulation
		canceled = false;
		// on supprime le Rdv
		ITask supprimerRvTask = (ITask) factory.getObject(Factory.SUPPRIMER_RV_TASK, this, "SupprimerRvTask");
		try {
			// on exécute la tâche (idRv)		
			supprimerRvTask.doWork(agenda.getCreneauxMedecinJour()[position].getRv().getId());
		} catch (RejectedExecutionException ex) {
			// la tâche a été rejetée
			super.notifyEvent(supprimerRvTask, IBoss.WORK_TERMINATED, (Object[]) null);
			// affichage exception
			showException(ex);
			// retour à l'UI
			return;
		}
		// début de l'attente
		beginWaiting();
		// début monitoring
		beginMonitoring();
	}

	// ajout d'un Rdv
	private void doAjouter(int position) {
		// on passe à la vue suivante
		Vue ajoutRvVue = (Vue) factory.getObject(Factory.AJOUT_RV_VUE, null, "AjoutRvVue");
		mainActivity.showVue(ajoutRvVue);
	}

	@Override
	public void notifyEndOfTasks() {
		// fin de l'attente
		cancelWaiting();
		// tâches annulées ?
		if (canceled) {
			return;
		}
		// on gère l'unique résultat
		Object result = results.get(0);
		// on traite les résultats des tâches exécutées
		if (result instanceof String) {
			// on nettoie la liste des résultats
			results.clear();
			// la suppression s'est bien passée - on régénère l'agenda
			ITask getAgendaMedecinTask = (ITask) factory.getObject(Factory.GETAGENDAMEDECIN_TASK, this,
					"AgendaMedecinJourTask");
			try {
				// on exécute la tâche (idMedecin, jourRv)
				getAgendaMedecinTask.doWork(agenda.getMedecin().getId(),
						new SimpleDateFormat("dd-MM-yyyy", Locale.FRANCE).format(agenda.getJour()));
			} catch (RejectedExecutionException ex) {
				// la tâche a été rejetée
				super.notifyEvent(getAgendaMedecinTask, IBoss.WORK_TERMINATED, (Object[]) null);
				// affichage exception
				showException(ex);
				// retour à l'UI
				return;
			}
			// début de l'attente
			beginWaiting();
			// début monitoring
			beginMonitoring();
		} else {
			if (result instanceof AgendaMedecinJour) {
				// on met l'agenda dans la session
				session.add("agenda", result);
				// on régénère la liste des créneaux
				AgendaMedecinJour agenda = (AgendaMedecinJour) result;
				ArrayAdapter<CreneauMedecinJour> adapter = new ListCreneauxAdapter(mainActivity, R.layout.creneau_medecin,
						agenda.getCreneauxMedecinJour(), this);
				lstCreneaux.setAdapter(adapter);
				// on se positionne au bon endroit du ListView
				lstCreneaux.setSelectionFromTop(firstPosition, top);
			} else if (result instanceof Exception) {
				// on affiche l'exception
				showException((Exception) result);
			}
		}
	}

	@Override
	// on reçoit des notifications de l'action
	public void notifyEvent(IWorker worker, int eventType, Object event) {
		// on passe l'info au parent
		super.notifyEvent(worker, eventType, event);
		// on traite le cas WORK_INFO
		if (eventType == IBoss.WORK_INFO) {
			results.add(event);
		}
	}

	// début de l'attente
	private void beginWaiting() {
		// on met le sablier
		showHourGlass();
		// état des boutons
		btnAnnuler.setVisibility(View.VISIBLE);
	}

	// fin de l'attente
	protected void cancelWaiting() {
		// on cache le sablier
		hideHourGlass();
		// état du bouton Annuler		
		btnAnnuler.setVisibility(View.INVISIBLE);
	}

}
