package android.rdvmedecins.ui.vues;

import istia.st.avat.android.Vue;
import istia.st.avat.core.IBoss;
import istia.st.avat.core.ITask;
import istia.st.avat.core.IWorker;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.concurrent.RejectedExecutionException;

import android.os.Bundle;
import android.rdvmedecins.entities.dao.Client;
import android.rdvmedecins.entities.dao.Creneau;
import android.rdvmedecins.entities.dao.Medecin;
import android.rdvmedecins.entities.dao.Rv;
import android.rdvmedecins.entities.metier.AgendaMedecinJour;
import android.rdvmedecins.ui.R;
import android.rdvmedecins.ui.activity.Factory;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;
import android.widget.TextView;

public class AjoutRvVue extends LocalVue {

	// les éléments de l'interface visuelle
	private Button btnValider;
	private Button btnAnnuler;
	private Spinner spinnerClients;
	private TextView txtTitre2;

	// les clients
	private List<Client> clients;

	// les résultats
	private List<Object> results = new ArrayList<Object>();

	// les saisies
	private String jour;
	private Long idCreneau;
	private Medecin medecin;
	private boolean canceled;

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		// on crée la vue du fragment à partir de sa définition XML
		return inflater.inflate(R.layout.ajout_rv, container, false);
	}

	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		// parent
		super.onActivityCreated(savedInstanceState);

		// titre
		txtTitre2 = (TextView) activity.findViewById(R.id.txt_titre2_ajoutRv);

		// spinner
		spinnerClients = (Spinner) activity.findViewById(R.id.spinnerClients);

		// bouton Valider
		btnValider = (Button) activity.findViewById(R.id.btn_Valider);
		btnValider.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View arg0) {
				doValider();
			}
		});

		// bouton Annuler
		btnAnnuler = (Button) activity.findViewById(R.id.btn_Annuler);
		btnAnnuler.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View arg0) {
				// on annule les tâches en cours
				cancelAll();
				// on annule l'attente
				cancelWaiting();
				// on note l'annulation
				canceled = true;
			}
		});
		// état des boutons
		btnValider.setVisibility(View.VISIBLE);
		btnAnnuler.setVisibility(View.INVISIBLE);
	}

	@SuppressWarnings("unchecked")
	@Override
	public void onResume() {
		// parent
		super.onResume();
		// on récupère les clients en session
		clients = (List<Client>) session.get("clients");
		// on construit le tableau affiché par le spinner
		String[] arrayClients = new String[clients.size()];
		int i = 0;
		for (Client client : clients) {
			arrayClients[i] = String.format("%s %s %s", client.getTitre(), client.getPrenom(), client.getNom());
			i++;
		}
		// on associe les clients au spinner
		ArrayAdapter<String> dataAdapter = new ArrayAdapter<String>(activity, android.R.layout.simple_spinner_item,
				arrayClients);
		dataAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		spinnerClients.setAdapter(dataAdapter);
		// on construit le titre 2 de la page
		// on récupère le n° du créneau à réserver en session
		int position = (Integer) session.get("position");
		// on récupère l'agenda du médecin dans la session
		AgendaMedecinJour agenda = (AgendaMedecinJour) session.get("agenda");
		medecin = agenda.getMedecin();
		Creneau creneau = agenda.getCreneauxMedecinJour()[position].getCreneau();
		idCreneau = creneau.getId();
		jour = new SimpleDateFormat("dd-MM-yyyy", Locale.FRANCE).format(agenda.getJour());
		String titre2 = String.format(Locale.FRANCE,
				"Prise de rendez-vous de %s %s %s le %s pour le créneau %02d:%02d-%02d:%02d", medecin.getTitre(),
				medecin.getPrenom(), medecin.getNom(), jour, creneau.getHdebut(), creneau.getMdebut(), creneau.getHfin(),
				creneau.getMfin());
		txtTitre2.setText(titre2);
		// on gère les liens de navigation
		activateLinks(3);
	}

	// validation de la page
	protected void doValider() {
		// on nettoie les résultats
		results.clear();
		// pas encore d'annulation
		canceled = false;
		// on récupère l'id du client choisi
		Long idClient = clients.get(spinnerClients.getSelectedItemPosition()).getId();
		// on ajoute le RV
		ITask ajouterRvTask = (ITask) factory.getObject(Factory.AJOUTER_RV_TASK, this, "AjouterRvTask");
		try {
			// on exécute la tâche (jour, idCreneau, idClient)
			ajouterRvTask.doWork(jour, idCreneau, idClient);
		} catch (RejectedExecutionException ex) {
			// la tâche a été rejetée
			super.notifyEvent(ajouterRvTask, IBoss.WORK_TERMINATED, (Object[]) null);
			// affichage exception
			showException(ex);
			// retour à l'UI
			return;
		}
		// début de l'attente
		beginWaiting();
		// début monitoring
		beginMonitoring();
	}

	@Override
	public void notifyEndOfTasks() {
		// fin de l'attente
		cancelWaiting();
		// tâches annulées ?
		if (canceled) {
			return;
		}
		// on gère le résultat unique
		Object result = results.get(0);
		if (result instanceof AgendaMedecinJour) {
			// on met l'agenda dans la session
			session.add("agenda", result);
			// on passe à la vue suivante
			Vue agendaVue = (Vue) factory.getObject(Factory.AGENDA_VUE, null, "AgendaVue");
			mainActivity.showVue(agendaVue);
		} else if (result instanceof Rv) {
			// on nettoie la liste des résultats
			results.clear();
			// on régénère l'agenda
			ITask getAgendaMedecinTask = (ITask) factory.getObject(Factory.GETAGENDAMEDECIN_TASK, this,
					"AgendaMedecinJourTask");
			try {
				// on exécute la tâche (idMedecin, jourRv)
				getAgendaMedecinTask.doWork(medecin.getId(), jour);
			} catch (RejectedExecutionException ex) {
				// la tâche a été rejetée
				super.notifyEvent(getAgendaMedecinTask, IBoss.WORK_TERMINATED, (Object[]) null);
				// affichage exception
				showException(ex);
				// retour à l'UI
				return;
			}
			// début de l'attente
			beginWaiting();
			// début monitoring
			beginMonitoring();
		} else if (result instanceof Exception) {
			// on affiche l'exception
			showException((Exception) result);
		}
	}

	@Override
	// on reçoit des notifications de l'action
	public void notifyEvent(IWorker worker, int eventType, Object event) {
		// on passe l'info au parent
		super.notifyEvent(worker, eventType, event);
		// on traite le cas WORK_INFO
		if (eventType == IBoss.WORK_INFO) {
			results.add(event);
		}
	}

	// début de l'attente
	private void beginWaiting() {
		// on met le sablier
		showHourGlass();
		// état des boutons
		btnValider.setVisibility(View.INVISIBLE);
		btnAnnuler.setVisibility(View.VISIBLE);

	}

	// fin de l'attente
	protected void cancelWaiting() {
		// on cache le sablier
		hideHourGlass();
		// état des boutons
		btnValider.setVisibility(View.VISIBLE);
		btnAnnuler.setVisibility(View.INVISIBLE);
	}

}
