package android.rdvmedecins.ui.vues;

import istia.st.avat.android.Vue;
import istia.st.avat.core.IBoss;
import istia.st.avat.core.ITask;
import istia.st.avat.core.IWorker;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.RejectedExecutionException;

import android.os.Bundle;
import android.rdvmedecins.ui.R;
import android.rdvmedecins.ui.activity.Factory;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

public class ConfigVue extends LocalVue {

	// les éléments de l'interface visuelle
	private Button btnValider;
	private Button btnAnnuler;
	private EditText edtUrlServiceRest;
	private TextView txtErrorUrlServiceRest;

	// ses résultats
	private List<Object> results = new ArrayList<Object>();

	// les saisies
	private String urlServiceRest;
	// à vrai s'il y a eu annulation
	private boolean canceled = false;

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		// on crée la vue du fragment à partir de sa définition XML
		return inflater.inflate(R.layout.config, container, false);
	}

	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		// parent
		super.onActivityCreated(savedInstanceState);

		// zones de saisie
		edtUrlServiceRest = (EditText) activity.findViewById(R.id.edt_urlServiceRest);

		// les messages d'erreur
		txtErrorUrlServiceRest = (TextView) activity.findViewById(R.id.txt_errorUrlServiceRest);

		// au départ pas de messages d'erreur
		txtErrorUrlServiceRest.setText("");

		// bouton Valider
		btnValider = (Button) activity.findViewById(R.id.btn_Valider);
		btnValider.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View arg0) {
				doValider();
			}
		});

		// bouton Annuler
		btnAnnuler = (Button) activity.findViewById(R.id.btn_Annuler);
		btnAnnuler.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View arg0) {
				// on annule les tâches en cours
				cancelAll();
				// on annule l'attente
				cancelWaiting();
				// on note l'annulation
				canceled = true;
			}
		});
		// état des boutons
		btnValider.setVisibility(View.VISIBLE);
		btnAnnuler.setVisibility(View.INVISIBLE);

	}

	@Override
	public void onResume() {
		// parent
		super.onResume();
		// on gère les liens de navigation
		activateLinks(0);
	}

	// validation de la page
	protected void doValider() {
		// on efface les éventuels msg d'erreur précédents
		txtErrorUrlServiceRest.setText("");
		// on teste la validité des saisies
		if (!isPageValid()) {
			return;
		}
		// on nettoie les résultats
		results.clear();
		// on n'a pas annulé
		canceled = false;

		// on demande la liste des médecins
		ITask getAllMedecinsTask = (ITask) factory.getObject(Factory.GETALLMEDECINS_TASK, this, "GetAllMedecinsTask");
		try {
			// on exécute la tâche (urlServiceRest)
			getAllMedecinsTask.doWork(edtUrlServiceRest.getText().toString());
		} catch (RejectedExecutionException ex) {
			// la tâche a été rejetée
			super.notifyEvent(getAllMedecinsTask, IBoss.WORK_TERMINATED, (Object[]) null);
			// affichage exception
			showException(ex);
			// retour à l'UI
			return;
		}
		// on demande la liste des clients
		ITask getAllClientsTask = (ITask) factory.getObject(Factory.GETALLCLIENTS_TASK, this, "GetAllClientsTask");
		try {
			// on exécute la tâche (urlServiceRest)
			getAllClientsTask.doWork(edtUrlServiceRest.getText().toString());
		} catch (RejectedExecutionException ex) {
			// la tâche a été rejetée
			super.notifyEvent(getAllMedecinsTask, IBoss.WORK_TERMINATED, (Object[]) null);
			// affichage exception
			showException(ex);
			// retour à l'UI
			return;
		}
		// début de l'attente
		beginWaiting();
		// début monitoring
		beginMonitoring();
	}

	private boolean isPageValid() {
		// on vérifie la validité des données saisies
		Boolean erreur = false;
		// validité de l'URL du service REST
		urlServiceRest = edtUrlServiceRest.getText().toString().trim();
		try {
			new URI(String.format("http://%s", urlServiceRest));
		} catch (Exception ex) {
			// on note l'erreur
			erreur = true;
			// affichage erreur
			txtErrorUrlServiceRest.setText(R.string.txt_errorUrlServiceRest);
		}
		// retour
		return !erreur;
	}

	@Override
	public void notifyEndOfTasks() {
		// fin de l'attente
		cancelWaiting();
		// on affiche le résultat
		Toast.makeText(activity, "Travail terminé", Toast.LENGTH_LONG).show();
		// tâches annulées ?
		if (canceled) {
			return;
		}
		boolean erreur = false;
		for (Object result : results) {
			// on traite les résultats des tâches exécutées
			if (result instanceof Map<?, ?>) {
				// on a un dictionnaire - son unique entrée est une liste - on la met dans la session
				@SuppressWarnings("unchecked")
				Map<String, Object> map = (Map<String, Object>) result;
				String clé = map.keySet().iterator().next();
				// on met la liste dans la session
				session.add(clé, map.get(clé));
			} else if (result instanceof Exception) {
				// on affiche l'exception
				showException((Exception) result);
				// on ne continue pas
				erreur = true;
				break;
			}
		}
		// si pas d'erreur, on passe à la vue suivante
		if (!erreur) {
			// on définit la première vue
			Vue accueilVue = (Vue) factory.getObject(Factory.ACCUEIL_VUE, null, "AccueilVue");
			// on fait de configVue la vue courante
			mainActivity.showVue(accueilVue);

		}
	}

	@Override
	// on reçoit des notifications de l'action
	public void notifyEvent(IWorker worker, int eventType, Object event) {
		// on passe l'info au parent
		super.notifyEvent(worker, eventType, event);
		// on traite le cas WORK_INFO
		if (eventType == IBoss.WORK_INFO) {
			results.add(event);
		}
	}

	// début de l'attente
	private void beginWaiting() {
		// on met le sablier
		showHourGlass();
		// état des boutons
		btnValider.setVisibility(View.INVISIBLE);
		btnAnnuler.setVisibility(View.VISIBLE);

	}

	// fin de l'attente
	protected void cancelWaiting() {
		// on cache le sablier
		hideHourGlass();
		// état des boutons
		btnValider.setVisibility(View.VISIBLE);
		btnAnnuler.setVisibility(View.INVISIBLE);
	}

}
