package android.rdvmedecins.ui.vues;

import java.util.Arrays;
import java.util.Comparator;

import android.app.Activity;
import android.content.Context;
import android.rdvmedecins.entities.dao.Client;
import android.rdvmedecins.entities.metier.CreneauMedecinJour;
import android.rdvmedecins.ui.R;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.TextView;

public class ListCreneauxAdapter extends ArrayAdapter<CreneauMedecinJour> {

	// le tableau des arduinos
	private CreneauMedecinJour[] creneauxMedecinJour;
	// le contexte d'exécution
	private Context context;
	// l'id du layout d'affichage d'une ligne de la liste des créneaux
	private int layoutResourceId;
	// listener des clics
	private AgendaVue vue;

	// constructeur
	public ListCreneauxAdapter(Context context, int layoutResourceId, CreneauMedecinJour[] creneauxMedecinJour,
			AgendaVue vue) {
		super(context, layoutResourceId, creneauxMedecinJour);
		// on mémorise les infos
		this.creneauxMedecinJour = creneauxMedecinJour;
		this.context = context;
		this.layoutResourceId = layoutResourceId;
		this.vue = vue;
		// on trie le tableau des créneaux dans l'ordre horaire
		Arrays.sort(creneauxMedecinJour, new MyComparator());
	}

	@Override
	public View getView(final int position, View convertView, ViewGroup parent) {
		// on se positionne sur le bon créneau
		CreneauMedecinJour creneauMedecin = creneauxMedecinJour[position];
		// on crée la ligne
		View row = ((Activity) context).getLayoutInflater().inflate(layoutResourceId, parent, false);
		// le créneau horaire
		TextView txtCreneau = (TextView) row.findViewById(R.id.txt_Creneau);
		txtCreneau.setText(String.format("%02d:%02d-%02d:%02d", creneauMedecin.getCreneau().getHdebut(), creneauMedecin
				.getCreneau().getMdebut(), creneauMedecin.getCreneau().getHfin(), creneauMedecin.getCreneau().getMfin()));
		// le client
		TextView txtClient = (TextView) row.findViewById(R.id.txt_Client);
		String text;
		if (creneauMedecin.getRv() != null) {
			Client client = creneauMedecin.getRv().getClient();
			text = String.format("%s %s %s", client.getTitre(), client.getPrenom(), client.getNom());
		} else {
			text = "";
		}
		txtClient.setText(text);
		// le bouton
		final TextView btnValider = (TextView) row.findViewById(R.id.btn_Valider);
		if (creneauMedecin.getRv() == null) {
			// ajouter
			btnValider.setText(R.string.btn_ajouter);
			btnValider.setTextColor(context.getResources().getColor(R.color.blue));
		} else {
			// supprimer
			btnValider.setText(R.string.btn_supprimer);
			btnValider.setTextColor(context.getResources().getColor(R.color.red));
		}
		// listener du bouton
		btnValider.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				// on passe les infos à la vue de l'agenda
				vue.doValider(position, btnValider.getText().toString());
			}
		});
		// on rend la ligne
		return row;
	}
}

// tri du tableau des créneaux
class MyComparator implements Comparator<CreneauMedecinJour> {

	@Override
	public int compare(CreneauMedecinJour creneau0, CreneauMedecinJour creneau1) {
		// on compare les 2 créneaux en tant que chaînes de caractères
		CreneauMedecinJour creneauMedecin = creneau0;
		String cr0 = String.format("%02d:%02d-%02d:%02d", creneauMedecin.getCreneau().getHdebut(), creneauMedecin
				.getCreneau().getMdebut(), creneauMedecin.getCreneau().getHfin(), creneauMedecin.getCreneau().getMfin());
		creneauMedecin = creneau1;
		String cr1 = String.format("%02d:%02d-%02d:%02d", creneauMedecin.getCreneau().getHdebut(), creneauMedecin
				.getCreneau().getMdebut(), creneauMedecin.getCreneau().getHfin(), creneauMedecin.getCreneau().getMfin());
		// résultat
		return cr0.compareTo(cr1);
	}
}