package android.rdvmedecins.ui.vues;

import istia.st.avat.android.Vue;
import istia.st.avat.core.ISession;
import android.app.DialogFragment;
import android.os.Bundle;
import android.rdvmedecins.ui.R;
import android.rdvmedecins.ui.activity.Factory;
import android.rdvmedecins.ui.activity.MainActivity;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.TextView;

public abstract class LocalVue extends Vue {
	// encapsule des méthodes et données communes aux classes filles
	protected ISession session;
	// la main activity
	protected MainActivity mainActivity;
	// navigation
	protected TextView lnkConfig;
	protected TextView lnkAccueil;
	protected TextView lnkAgenda;
	protected TextView lnkAjout;
	protected TextView[] links;

	// constructeur
	public LocalVue() {
	}

	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		// parent
		super.onActivityCreated(savedInstanceState);
		// la session
		session = (ISession) factory.getObject(Factory.SESSION, (Object[]) null);
		// la main activity
		mainActivity = (MainActivity) activity;
		// les liens de la navigation
		lnkConfig = (TextView) activity.findViewById(R.id.lnk_Config);
		lnkAccueil = (TextView) activity.findViewById(R.id.lnk_Accueil);
		lnkAgenda = (TextView) activity.findViewById(R.id.lnk_Agenda);
		lnkAjout = (TextView) activity.findViewById(R.id.lnk_Ajout);
		// tableau des liens
		links = new TextView[] { lnkConfig, lnkAccueil, lnkAgenda, lnkAjout };
	}

	// navigation
	protected void navigateToView(View v) {
		// libellé du lien
		String libellé = ((TextView) v).getText().toString();
		// affichage de la vue correspondant au lien
		if (libellé.equals(getResources().getString(R.string.lnk_config))) {
			mainActivity.showVue((Vue) factory.getObject(Factory.CONFIG_VUE, (Object[]) null));
		}
		if (libellé.equals(getResources().getString(R.string.lnk_accueil))) {
			mainActivity.showVue((Vue) factory.getObject(Factory.ACCUEIL_VUE, (Object[]) null));
		}
		if (libellé.equals(getResources().getString(R.string.lnk_agenda))) {
			mainActivity.showVue((Vue) factory.getObject(Factory.AGENDA_VUE, (Object[]) null));
		}
		if (libellé.equals(getResources().getString(R.string.lnk_ajout))) {
			mainActivity.showVue((Vue) factory.getObject(Factory.AJOUT_RV_VUE, (Object[]) null));
		}
	}

	// gestion des liens
	protected void activateLinks(int idLink) {
		// on active tous les liens qui précèdent idLink
		int i = 0;
		while (i < idLink) {
			links[i].setTextColor(getResources().getColor(R.color.blue));
			links[i].setOnClickListener(new OnClickListener() {

				@Override
				public void onClick(View v) {
					navigateToView(v);

				}
			});
			i++;
		}
		// on désactive le lien lnk et les suivants
		while (i < links.length) {
			links[i].setTextColor(getResources().getColor(R.color.black));
			links[i].setOnClickListener(null);
			i++;
		}
	}

	// affichage exception
	protected void showException(Exception ex) {
		// on affiche l'exception dans une boîte de dialogue
		DialogFragment dialog = new ExceptionVue(ex);
		dialog.show(getFragmentManager(), "exception");
	}

	// gestion du sablier
	protected void showHourGlass() {
		activity.setProgressBarIndeterminateVisibility(true);
	}

	protected void hideHourGlass() {
		activity.setProgressBarIndeterminateVisibility(false);
	}

	@Override
	abstract public void notifyEndOfTasks();

}
