package server.rdvmedecins.rest;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.View;

import server.rdvmedecins.entities.jpa.Client;
import server.rdvmedecins.entities.jpa.Creneau;
import server.rdvmedecins.entities.jpa.Medecin;
import server.rdvmedecins.entities.jpa.Rv;
import server.rdvmedecins.entities.metier.AgendaMedecinJour;
import server.rdvmedecins.metier.service.IMetier;

@Controller
public class Rest {

  // couche métier
  @Autowired
  private IMetier metier;
  @Autowired
  private View view;

  // liste des clients
  @RequestMapping(value = "/getAllClients", method = RequestMethod.GET)
  public ModelAndView getAllClients() {

    // on utilise la couche métier
    try {
      List<Client> clients = metier.getAllClients();
      // réponse
      return new ModelAndView(view, "data", clients);
    } catch (Exception e) {
      // réponse
      return createResponseError("103", e.getMessage());
    }
  }

  // liste des médecins
  @RequestMapping(value = "/getAllMedecins", method = RequestMethod.GET)
  public ModelAndView getAllMedecins() {

    // on utilise la couche métier
    try {
      List<Medecin> medecins = metier.getAllMedecins();
      // réponse
      return new ModelAndView(view, "data", medecins);
    } catch (Exception e) {
      // réponse
      return createResponseError("104", e.getMessage());
    }
  }

  // liste des créneaux horaires d'un médecin
  @RequestMapping(value = "/getAllCreneaux/{idMedecin}", method = RequestMethod.GET)
  public ModelAndView getAllCreneaux(@PathVariable("idMedecin") long idMedecin) {

    // on utilise la couche métier
    try {
      Medecin medecin = metier.getMedecinById(idMedecin);
      List<Creneau> creneaux = metier.getAllCreneaux(medecin);
      // réponse
      return new ModelAndView(view, "data", creneaux);
    } catch (Exception e) {
      // réponse
      return createResponseError("105", e.getMessage());
    }
  }

  // liste des Rv d'un médecin, un jour donné
  @RequestMapping(value = "/getRvMedecinJour/{idMedecin}/{jour}", method = RequestMethod.GET)
  public ModelAndView getRvMedecinJour(@PathVariable("idMedecin") long idMedecin, @PathVariable("jour") String jour) {

    // on utilise la couche métier
    try {
      Date jourRv = new SimpleDateFormat("dd-MM-yy").parse(jour);
      Medecin medecin = metier.getMedecinById(idMedecin);
      List<Rv> rv = metier.getRvMedecinJour(medecin, jourRv);
      // réponse
      return new ModelAndView(view, "data", rv);
    } catch (Exception e) {
      // réponse
      return createResponseError("106", e.getMessage());
    }
  }

  // trouver un client identifié par son id
  @RequestMapping(value = "/getClientById/{idClient}", method = RequestMethod.GET)
  public ModelAndView getClientById(@PathVariable("idClient") long idClient) {

    // on utilise la couche métier
    try {
      // réponse
      return new ModelAndView(view, "data", metier.getClientById(idClient));
    } catch (Exception e) {
      // réponse
      return createResponseError("107", e.getMessage());
    }
  }

  // trouver un médecin identifié par son id
  @RequestMapping(value = "/getMedecinById/{idMedecin}", method = RequestMethod.GET)
  public ModelAndView getMedecinById(@PathVariable("idMedecin") long idMedecin) {

    // on utilise la couche métier
    try {
      // réponse
      return new ModelAndView(view, "data", metier.getMedecinById(idMedecin));
    } catch (Exception e) {
      // réponse
      return createResponseError("108", e.getMessage());
    }
  }

  // trouver un Rv identifié par son id
  @RequestMapping(value = "/getRvById/{idRv}", method = RequestMethod.GET)
  public ModelAndView getRvById(@PathVariable("idRv") long idRv) {

    // on utilise la couche métier
    try {
      // réponse
      return new ModelAndView(view, "data", metier.getRvById(idRv));
    } catch (Exception e) {
      // réponse
      return createResponseError("109", e.getMessage());
    }
  }

  // trouver un créneau horaire identifié par son id
  @RequestMapping(value = "/getCreneauById/{idCreneau}", method = RequestMethod.GET)
  public ModelAndView getCreneauById(@PathVariable("idCreneau") long idCreneau) {

    // on utilise la couche métier
    try {
      // réponse
      return new ModelAndView(view, "data", metier.getCreneauById(idCreneau));
    } catch (Exception e) {
      // réponse
      return createResponseError("110", e.getMessage());
    }
  }

  // ajouter un RV
  @RequestMapping(value = "/ajouterRv/{jour}/{idCreneau}/{idClient}", method = RequestMethod.GET)
  public ModelAndView ajouterRv(@PathVariable("jour") String jour, @PathVariable("idCreneau") long idCreneau,
          @PathVariable("idClient") long idClient) {

    // on utilise la couche métier
    try {
      Date jourRv = new SimpleDateFormat("dd-MM-yyyy").parse(jour);
      Client client = metier.getClientById(idClient);
      Creneau creneau = metier.getCreneauById(idCreneau);
      Rv rv = metier.ajouterRv(jourRv, creneau, client);
      // réponse
      return new ModelAndView(view, "data", rv);
    } catch (Exception e) {
      // réponse
      return createResponseError("111", e.getMessage());
    }
  }

  // supprimer un RV
  @RequestMapping(value = "/supprimerRv/{idRv}", method = RequestMethod.GET)
  public ModelAndView supprimerRv(@PathVariable("idRv") long idRv) {

    // on utilise la couche métier
    try {
      Rv rv = metier.getRvById(idRv);
      metier.supprimerRv(rv);
      // réponse
      return new ModelAndView(view, "data", "OK");
    } catch (Exception e) {
      // réponse
      return createResponseError("112", e.getMessage());
    }
  }

  // l'agenda d'un medecin
  @RequestMapping(value = "/getAgendaMedecinJour/{idMedecin}/{jour}", method = RequestMethod.GET)
  public ModelAndView getAgendaMedecinJour(@PathVariable("idMedecin") long idMedecin, @PathVariable("jour") String jour) {

    // on utilise la couche métier
    try {
      Date jourRv = new SimpleDateFormat("dd-MM-yyyy").parse(jour);
      Medecin medecin = metier.getMedecinById(idMedecin);
      AgendaMedecinJour agenda = metier.getAgendaMedecinJour(medecin, jourRv);
      System.out.println(String.format("jourRv=%s, jour=%s",jourRv, agenda.getJour()));
      // réponse
      return new ModelAndView(view, "data", agenda);
    } catch (Exception e) {
      // réponse
      return createResponseError("113", e.getMessage());
    }
  }

  // crée une réponse d'erreur
  private ModelAndView createResponseError(String numero, String message) {
    Map<String, Object> modèle = new HashMap<String, Object>();
    modèle.put("error", numero);
    modèle.put("message", message);
    return new ModelAndView(view, modèle);
  }
  
}
