package android.rdvmedecins.metier;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import android.rdvmedecins.dao.IDao;
import android.rdvmedecins.entities.dao.Client;
import android.rdvmedecins.entities.dao.Creneau;
import android.rdvmedecins.entities.dao.Medecin;
import android.rdvmedecins.entities.dao.Rv;
import android.rdvmedecins.entities.exceptions.RdvMedecinsException;
import android.rdvmedecins.entities.metier.AgendaMedecinJour;
import android.rdvmedecins.entities.metier.CreneauMedecinJour;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

public class Metier implements IMetier {

	// couche [dao]
	private IDao dao;
	// mapper JSON
	private Gson gson = new Gson();
	// url du service REST
	private String urlServiceRest;

	// @RequestMapping(value = "/getAllClients", method = RequestMethod.GET)
	/*
	 * (non-Javadoc)
	 * 
	 * @see android.rdvmedecins.metier.IMetier#getAllClients()
	 */
	public List<Client> getAllClients() {
		// adresse du service REST
		String urlService = String.format("http://%s/getAllClients", urlServiceRest);
		// exécution service
		String réponse = dao.executeRestService("get", urlService, null, new HashMap<String, String>());
		// exploitation réponse
		try {
			return gson.fromJson(réponse, new TypeToken<List<Client>>() {
			}.getType());
		} catch (Exception ex) {
			throw new RdvMedecinsException(String.format("Réponse incorrecte du serveur: %s", réponse), ex, 20);
		}
	}

	// @RequestMapping(value = "/getAllMedecins", method = RequestMethod.GET)
	/*
	 * (non-Javadoc)
	 * 
	 * @see android.rdvmedecins.metier.IMetier#getAllMedecins()
	 */
	public List<Medecin> getAllMedecins() {
		// adresse du service REST
		String urlService = String.format("http://%s/getAllMedecins", urlServiceRest);
		// exécution service
		String réponse = dao.executeRestService("get", urlService, null, new HashMap<String, String>());
		// exploitation réponse
		try {
			return gson.fromJson(réponse, new TypeToken<List<Medecin>>() {
			}.getType());
		} catch (Exception ex) {
			throw new RdvMedecinsException(String.format("Réponse incorrecte du serveur: %s", réponse), ex, 21);
		}
	}

	// @RequestMapping(value = "/getAllCreneaux/{idMedecin}", method =
	// RequestMethod.GET)
	/*
	 * (non-Javadoc)
	 * 
	 * @see android.rdvmedecins.metier.IMetier#getAllCreneaux(java.lang.Long)
	 */
	public List<Creneau> getAllCreneaux(Long idMedecin) {
		// adresse du service REST
		String urlService = String.format("http://%s/getAllCreneaux/{idMedecin}", urlServiceRest);
		// paramètres du service REST
		Map<String, String> paramètres = new HashMap<String, String>();
		paramètres.put("idMedecin", String.valueOf(idMedecin));
		// exécution service
		String réponse = dao.executeRestService("get", urlService, null, paramètres);
		// exploitation réponse
		try {
			return gson.fromJson(réponse, new TypeToken<List<Creneau>>() {
			}.getType());
		} catch (Exception ex) {
			throw new RdvMedecinsException(String.format("Réponse incorrecte du serveur: %s", réponse), ex, 22);
		}
	}

	// @RequestMapping(value = "/getRvMedecinJour/{idMedecin}/{jour}", method =
	// RequestMethod.GET)
	/*
	 * (non-Javadoc)
	 * 
	 * @see android.rdvmedecins.metier.IMetier#getRvMedecinJour(java.lang.Long,
	 * java.lang.String)
	 */
	public List<Rv> getRvMedecinJour(Long idMedecin, String jour) {
		// adresse du service REST
		String urlService = String.format("http://%s/getRvMedecinJour/{idMedecin}/{jour}", urlServiceRest);
		// paramètres du service REST
		Map<String, String> paramètres = new HashMap<String, String>();
		paramètres.put("idMedecin", String.valueOf(idMedecin));
		paramètres.put("jour", jour);
		// exécution service
		String réponse = dao.executeRestService("get", urlService, null, paramètres);
		// exploitation réponse
		try {
			return gson.fromJson(réponse, new TypeToken<List<Rv>>() {
			}.getType());
		} catch (Exception ex) {
			throw new RdvMedecinsException(String.format("Réponse incorrecte du serveur: %s", réponse), ex, 23);
		}
	}

	// @RequestMapping(value = "/getClientById/{idClient}", method =
	// RequestMethod.GET)
	/*
	 * (non-Javadoc)
	 * 
	 * @see android.rdvmedecins.metier.IMetier#getClientById(java.lang.Long)
	 */
	public Client getClientById(Long idClient) {
		// adresse du service REST
		String urlService = String.format("http://%s/getClientById/{idClient}", urlServiceRest);
		// paramètres du service REST
		Map<String, String> paramètres = new HashMap<String, String>();
		paramètres.put("idClient", String.valueOf(idClient));
		// exécution service
		String réponse = dao.executeRestService("get", urlService, null, paramètres);
		// exploitation réponse
		try {
			return gson.fromJson(réponse, new TypeToken<Client>() {
			}.getType());
		} catch (Exception ex) {
			throw new RdvMedecinsException(String.format("Réponse incorrecte du serveur: %s", réponse), ex, 23);
		}
	}

	// @RequestMapping(value = "/getMedecinById/{idMedecin}", method =
	// RequestMethod.GET)
	/*
	 * (non-Javadoc)
	 * 
	 * @see android.rdvmedecins.metier.IMetier#getMedecinById(java.lang.Long)
	 */
	public Medecin getMedecinById(Long idMedecin) {
		// adresse du service REST
		String urlService = String.format("http://%s/getMedecinById/{idMedecin}", urlServiceRest);
		// paramètres du service REST
		Map<String, String> paramètres = new HashMap<String, String>();
		paramètres.put("idMedecin", String.valueOf(idMedecin));
		// exécution service
		String réponse = dao.executeRestService("get", urlService, null, paramètres);
		// exploitation réponse
		try {
			return gson.fromJson(réponse, new TypeToken<Medecin>() {
			}.getType());
		} catch (Exception ex) {
			throw new RdvMedecinsException(String.format("Réponse incorrecte du serveur: %s", réponse), ex, 24);
		}
	}

	// @RequestMapping(value = "/getRvById/{idRv}", method = RequestMethod.GET)
	/*
	 * (non-Javadoc)
	 * 
	 * @see android.rdvmedecins.metier.IMetier#getRvById(java.lang.Long)
	 */
	public Rv getRvById(Long idRv) {
		// adresse du service REST
		String urlService = String.format("http://%s/getRvById/{idRv}", urlServiceRest);
		// paramètres du service REST
		Map<String, String> paramètres = new HashMap<String, String>();
		paramètres.put("idRv", String.valueOf(idRv));
		// exécution service
		String réponse = dao.executeRestService("get", urlService, null, paramètres);
		// exploitation réponse
		try {
			return gson.fromJson(réponse, new TypeToken<Rv>() {
			}.getType());
		} catch (Exception ex) {
			throw new RdvMedecinsException(String.format("Réponse incorrecte du serveur: %s", réponse), ex, 25);
		}
	}

	// @RequestMapping(value = "/getCreneauById/{idCreneau}", method =
	// RequestMethod.GET)
	/*
	 * (non-Javadoc)
	 * 
	 * @see android.rdvmedecins.metier.IMetier#getCreneauById(java.lang.Long)
	 */
	public Creneau getCreneauById(Long idCreneau) {
		// adresse du service REST
		String urlService = String.format("http://%s/getCreneauById/{idCreneau}", urlServiceRest);
		// paramètres du service REST
		Map<String, String> paramètres = new HashMap<String, String>();
		paramètres.put("idCreneau", String.valueOf(idCreneau));
		// exécution service
		String réponse = dao.executeRestService("get", urlService, null, paramètres);
		// exploitation réponse
		try {
			return gson.fromJson(réponse, new TypeToken<Creneau>() {
			}.getType());
		} catch (Exception ex) {
			throw new RdvMedecinsException(String.format("Réponse incorrecte du serveur: %s", réponse), ex, 26);
		}
	}

	// @RequestMapping(value = "/ajouterRv/{jour}/{idCreneau}/{idClient}", method
	// = RequestMethod.GET)
	/*
	 * (non-Javadoc)
	 * 
	 * @see android.rdvmedecins.metier.IMetier#ajouterRv(java.lang.String,
	 * java.lang.Long, java.lang.Long)
	 */
	public Rv ajouterRv(String jour, Long idCreneau, Long idClient) {
		// adresse du service REST
		String urlService = String.format("http://%s/ajouterRv/{jour}/{idCreneau}/{idClient}", urlServiceRest);
		// paramètres du service REST
		Map<String, String> paramètres = new HashMap<String, String>();
		paramètres.put("idCreneau", String.valueOf(idCreneau));
		paramètres.put("idClient", String.valueOf(idClient));
		paramètres.put("jour", jour);
		// exécution service
		String réponse = dao.executeRestService("get", urlService, null, paramètres);
		// exploitation réponse
		try {
			// on récupère une map car on a un pb avec la sérialisation de la date au format JSon
			Map<String, Object> map = gson.fromJson(réponse, new TypeToken<Map<String, Object>>() {
			}.getType());
			// on rend le résultat
			return getRvFromMap(map, jour);
		} catch (Exception ex) {
			throw new RdvMedecinsException(String.format("Réponse incorrecte du serveur: %s", réponse), ex, 27);
		}
	}

	private Rv getRvFromMap(Map<String, Object> map, String jour) throws ParseException {
		// on construit un objet [Rv] avec la map
		Rv rv = new Rv();
		rv.setJour(new SimpleDateFormat("dd-MM-yyyy").parse(jour));
		rv.setClient(gson.fromJson(gson.toJson(map.get("client")), Client.class));
		rv.setCreneau(gson.fromJson(gson.toJson(map.get("creneau")), Creneau.class));
		rv.setId(gson.fromJson(gson.toJson(map.get("id")), Long.class));
		// on rend le résultat
		return rv;
	}

	// @RequestMapping(value = "/supprimerRv/{idRv}", method = RequestMethod.GET)
	/*
	 * (non-Javadoc)
	 * 
	 * @see android.rdvmedecins.metier.IMetier#supprimerRv(java.lang.Long)
	 */
	public void supprimerRv(Long idRv) {
		// adresse du service REST
		String urlService = String.format("http://%s/supprimerRv/{idRv}", urlServiceRest);
		// paramètres du service REST
		Map<String, String> paramètres = new HashMap<String, String>();
		paramètres.put("idRv", String.valueOf(idRv));
		// exécution service
		dao.executeRestService("get", urlService, null, paramètres);
	}

	// @RequestMapping(value = "/getAgendaMedecinJour/{idMedecin}/{jour}", method
	// = RequestMethod.GET)
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * android.rdvmedecins.metier.IMetier#getAgendaMedecinJour(java.lang.Long,
	 * java.lang.String)
	 */
	public AgendaMedecinJour getAgendaMedecinJour(Long idMedecin, String jour) {
		// adresse du service REST
		String urlService = String.format("http://%s/getAgendaMedecinJour/{idMedecin}/{jour}", urlServiceRest);
		// paramètres du service REST
		Map<String, String> paramètres = new HashMap<String, String>();
		paramètres.put("idMedecin", String.valueOf(idMedecin));
		paramètres.put("jour", jour);
		// exécution service
		String réponse = dao.executeRestService("get", urlService, null, paramètres);
		// exploitation réponse
		try {
			// on récupère une map incorrecte car on a un pb avec la sérialisation de la date au format JSon
			Map<String, Object> map = gson.fromJson(réponse, new TypeToken<Map<String, Object>>() {
			}.getType());
			// on rend l'agenda à partir de cette map
			return getAgendaFromMap(map, jour);
		} catch (Exception ex) {
			throw new RdvMedecinsException(String.format("Réponse incorrecte du serveur: %s", réponse), ex, 28);
		}
	}

	private AgendaMedecinJour getAgendaFromMap(Map<String, Object> map, String jour) throws ParseException {
		// on a un souci avec le jour. Le serveur la sérialise d'une façon non reconnue par ce client
		// on est obligés de construire l'agenda nous-mêmes

		// on construit un objet [AgendaMedecinJour]
		AgendaMedecinJour agenda = new AgendaMedecinJour();
		agenda.setJour(new SimpleDateFormat("dd-MM-yyyy").parse(jour));
		agenda.setMedecin(gson.fromJson(gson.toJson(map.get("medecin")), Medecin.class));
		// les RV ont également un champ jour
		@SuppressWarnings("unchecked")
		List<Map<String,Object>> creneaux = (List<Map<String,Object>>) map.get("creneauxMedecinJour");
		CreneauMedecinJour[] creneauxMedecinJour = new CreneauMedecinJour[creneaux.size()];
		for (int i = 0; i < creneaux.size(); i++) {
			// un nouveau créneau
			creneauxMedecinJour[i] = new CreneauMedecinJour();
			creneauxMedecinJour[i].setCreneau(gson.fromJson(gson.toJson(creneaux.get(i).get("creneau")), Creneau.class));
			// un Rv ?
			@SuppressWarnings("unchecked")
			Map<String, Object> rvMap = (Map<String, Object>) creneaux.get(i).get("rv");
			Rv rv = null;
			if (rvMap != null) {
				// on construit le Rv
				rv = new Rv();
				rv.setJour(agenda.getJour());
				rv.setClient(gson.fromJson(gson.toJson(rvMap.get("client")), Client.class));
				rv.setId(gson.fromJson(gson.toJson(rvMap.get("id")), Long.class));
				rv.setCreneau(gson.fromJson(gson.toJson(rvMap.get("creneau")), Creneau.class));
			}
			creneauxMedecinJour[i].setRv(rv);
		}
		agenda.setCreneauxMedecinJour(creneauxMedecinJour);
		// résultat		
		return agenda;
	}

	// setters

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * android.rdvmedecins.metier.IMetier#setDao(android.rdvmedecins.dao.IDao)
	 */
	public void setDao(IDao dao) {
		this.dao = dao;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.rdvmedecins.metier.IMetier#setUrlServiceRest(java.lang.String)
	 */
	public void setUrlServiceRest(String urlServiceRest) {
		this.urlServiceRest = urlServiceRest;
	}
}
